@@ -16,6 +16,8 @@
 #define ISRHEL7
 #elif RHEL_MAJOR == 8
 #define ISRHEL8
+#elif RHEL_MAJOR == 9
+#define ISRHEL9
 #endif
 #endif
 #ifdef UTS_UBUNTU_RELEASE_ABI
@@ -27,6 +29,8 @@
 #define ISUBUNTU1904
 #elif LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(5, 3, 0)
 #define ISUBUNTU1910
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0)
+#define ISUBUNTU2204
 #endif
 #endif
 
@@ -534,7 +538,7 @@ static inline void *__compat_kvcalloc(size_t n, size_t size, gfp_t flags)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0)
-#define wg_newlink(a,b,c,d,e) wg_newlink(a,b,c,d)
+#define wg_newlink_old(a,b,c,d,e) wg_newlink_old(a,b,c,d)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 12, 0)
@@ -868,7 +872,7 @@ static inline void skb_mark_not_on_list(struct sk_buff *skb)
 #endif
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0) && !defined(ISRHEL8)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0) && !defined(ISRHEL8) && !defined(ISRHEL9)
 #define genl_dumpit_info(cb) ({ \
 	struct { struct nlattr **attrs; } *a = (void *)((u8 *)cb->args + offsetofend(struct dump_ctx, next_allowedip)); \
 	BUILD_BUG_ON(sizeof(cb->args) < offsetofend(struct dump_ctx, next_allowedip) + sizeof(*a)); \
@@ -1126,6 +1130,7 @@ static const struct header_ops ip_tunnel_header_ops = { .parse_protocol = ip_tun
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 30)
+#define COMPAT_HAS_DEFINED_DST_CACHE_PCPU
 #include <net/dst_cache.h>
 struct dst_cache_pcpu {
 	unsigned long refresh_ts;
@@ -1136,7 +1141,12 @@ struct dst_cache_pcpu {
 		struct in6_addr in6_saddr;
 	};
 };
-#define COMPAT_HAS_DEFINED_DST_CACHE_PCPU
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0) && \
+    !(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 7) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0)) && \
+    !(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 84) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)) && \
+    !defined(ISRHEL9)
 static inline void dst_cache_reset_now(struct dst_cache *dst_cache)
 {
 	int i;
@@ -1200,13 +1210,14 @@ static inline void dst_cache_reset_now(struct dst_cache *dst_cache)
 #define from_timer(var, callback_timer, timer_fieldname) container_of((struct timer_list *)callback_timer, typeof(*var), timer_fieldname)
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0) && \
+    !(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 121) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0))
 #include <net/flow.h>
 #define flowi4_to_flowi_common(fl4) flowi4_to_flowi(fl4)
 #define flowi6_to_flowi_common(fl4) flowi6_to_flowi(fl4)
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0) && !defined(ISRHEL9)
 #define genl_info_dump(cb) genl_dumpit_info(cb)
 #endif
 
@@ -1218,7 +1229,7 @@ static inline void dst_cache_reset_now(struct dst_cache *dst_cache)
 #define timer_delete_sync(timer) del_timer_sync(timer)
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 4) && !defined(ISRHEL9)
 #include <linux/random.h>
 static inline u32 get_random_u32_below(u32 ceil)
 {
@@ -1230,7 +1241,7 @@ static inline u32 get_random_u32_inclusive(u32 floor, u32 ceil)
 }
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0) && !defined(ISRHEL9)
 #define COMPAT_NETIF_HAS_WEIGHT
 #endif
 
@@ -1238,7 +1249,7 @@ static inline u32 get_random_u32_inclusive(u32 floor, u32 ceil)
 #define COMPAT_GENL_HAS_RESV_START_OP
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0) && \
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 2) && \
 	!(LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 296) && LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0)) && \
 	!(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 229) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0)) && \
 	!(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 163) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)) && \
@@ -1249,7 +1260,7 @@ static inline u32 get_random_u32_inclusive(u32 floor, u32 ceil)
 #define DEV_STATS_ADD(DEV, FIELD, VAL) DEV->stats.FIELD += VAL
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 17, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 17, 0) && !defined(ISRHEL9)
 #define COMPAT_SKB_HAS_SKB_START
 #endif
 
@@ -1257,11 +1268,13 @@ static inline u32 get_random_u32_inclusive(u32 floor, u32 ceil)
 #define dev_get_tstats64 ip_tunnel_get_stats64
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 12, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 12, 0) || defined(ISRHEL9)
 #define COMPAT_NETDEV_HAS_LLTX_PARAM
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
+#include <linux/if.h>
+#include <linux/if_tunnel.h>
 static inline void dev_sw_netstats_rx_add(struct net_device *dev, unsigned int len) {
 	struct pcpu_sw_netstats *tstats = get_cpu_ptr(dev->tstats);
 
@@ -1273,4 +1286,115 @@ static inline void dev_sw_netstats_rx_add(struct net_device *dev, unsigned int l
 }
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 91) && !defined(ISUBUNTU2204) && !defined(ISRHEL9)
+#include <linux/timer.h>
+static inline int timer_delete(struct timer_list *timer)
+{
+	return del_timer(timer);
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 16, 0)
+#define timer_container_of from_timer
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 17, 0)
+#include <linux/in6.h>
+struct sockaddr_inet {
+	unsigned short	sa_family;
+	char		sa_data[sizeof(struct sockaddr_in6) -
+				sizeof(unsigned short)];
+};
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 17, 0)
+#include <linux/netdevice.h>
+static inline void netif_threaded_enable(struct net_device *dev) { }
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 0, 0)
+#undef CONFIG_PM_USERSPACE_AUTOSLEEP
+#define CONFIG_PM_USERSPACE_AUTOSLEEP CONFIG_ANDROID
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 7, 0)
+#define COMPAT_CANNOT_USE_PCPU_STAT_TYPE
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 15, 0)
+#define COMPAT_CANNOT_USE_RTNL_NEWLINK_PARAMS
+struct rtnl_newlink_params {
+	struct net *src_net;
+	struct net *link_net;
+	struct net *peer_net;
+	struct nlattr **tb;
+	struct nlattr **data;
+};
+static inline struct net *rtnl_newlink_link_net(struct rtnl_newlink_params *p)
+{
+	return p->link_net ? : p->src_net;
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0) && !defined(ISRHEL9)
+#include <linux/notifier.h>
+static inline int register_random_vmfork_notifier(struct notifier_block *nb) { return 0; }
+static inline int unregister_random_vmfork_notifier(struct notifier_block *nb) { return 0; }
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0) && !defined(ISRHEL9)
+#include <linux/netdevice.h>
+static inline void netif_set_tso_max_size(struct net_device *dev, unsigned int size) {}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0)
+#define __nonstring
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0)
+static inline bool skb_queue_empty_lockless(const struct sk_buff_head *list)
+{
+	return READ_ONCE(list->next) == (const struct sk_buff *) list;
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 10, 0)
+#define NLA_POLICY_MASK(tp, _mask) { .type = tp }
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 14, 0)
+static inline char *nla_strdup(const struct nlattr *nla, gfp_t flags)
+{
+	size_t srclen = nla_len(nla);
+	char *src = nla_data(nla), *dst;
+
+	if (srclen > 0 && src[srclen - 1] == '\0')
+		srclen--;
+
+	dst = kmalloc(srclen + 1, flags);
+	if (dst != NULL) {
+		memcpy(dst, src, srclen);
+		dst[srclen] = '\0';
+	}
+	return dst;
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 17, 0)
+#include <linux/time.h>
+#define ktime_get_real_seconds get_seconds
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 12, 0)
+#include <asm/unaligned.h>
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 13, 0)
+#define genlmsg_multicast_netns(a, b, c, d, e, f) genlmsg_multicast_netns(b, c, d, e, f)
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 13, 0)
+#define COMPAT_CANNOT_USE_NETLINK_MCGRPS
+#endif
+
 #endif /* _WG_COMPAT_H */
